@@ -49,12 +49,11 @@
 #include "ednssubnet.hh"
 #include "query-local-address.hh"
 
-#ifdef HAVE_PROTOBUF
-
+#include "rec-protozero.hh"
 #include "uuid-utils.hh"
 
 #ifdef HAVE_FSTRM
-#include "rec-dnstap.hh"
+#include "dnstap.hh"
 #include "fstrm_logger.hh"
 
 
@@ -73,17 +72,15 @@ static bool isEnabledForQueries(const std::shared_ptr<std::vector<std::unique_pt
   return false;
 }
 
-static void logFstreamQuery(const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstreamLoggers, const struct timeval &queryTime, const ComboAddress& ip, bool doTCP,
-  boost::optional<const DNSName&> auth, const vector<uint8_t>& packet)
+static void logFstreamQuery(const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstreamLoggers, const struct timeval &queryTime, const ComboAddress& localip, const ComboAddress& ip, bool doTCP, boost::optional<const DNSName&> auth, const vector<uint8_t>& packet)
 {
   if (fstreamLoggers == nullptr)
     return;
 
   struct timespec ts;
   TIMEVAL_TO_TIMESPEC(&queryTime, &ts);
-  RecDnstapMessage message(SyncRes::s_serverID, nullptr, &ip, doTCP, auth, reinterpret_cast<const char*>(&*packet.begin()), packet.size(), &ts, nullptr);
   std::string str;
-  message.serialize(str);
+  DnstapMessage message(str, DnstapMessage::MessageType::resolver_query, SyncRes::s_serverID, &localip, &ip, doTCP, reinterpret_cast<const char*>(&*packet.begin()), packet.size(), &ts, nullptr, auth);
 
   for (auto& logger : *fstreamLoggers) {
     logger->queueData(str);
@@ -103,8 +100,7 @@ static bool isEnabledForResponses(const std::shared_ptr<std::vector<std::unique_
   return false;
 }
 
-static void logFstreamResponse(const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstreamLoggers, const ComboAddress& ip, bool doTCP,
-  boost::optional<const DNSName&> auth, const std::string& packet, const struct timeval& queryTime, const struct timeval& replyTime)
+static void logFstreamResponse(const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstreamLoggers, const ComboAddress&localip, const ComboAddress& ip, bool doTCP, boost::optional<const DNSName&> auth, const std::string& packet, const struct timeval& queryTime, const struct timeval& replyTime)
 {
   if (fstreamLoggers == nullptr)
     return;
@@ -112,9 +108,8 @@ static void logFstreamResponse(const std::shared_ptr<std::vector<std::unique_ptr
   struct timespec ts1, ts2;
   TIMEVAL_TO_TIMESPEC(&queryTime, &ts1);
   TIMEVAL_TO_TIMESPEC(&replyTime, &ts2);
-  RecDnstapMessage message(SyncRes::s_serverID, nullptr, &ip, doTCP, auth, static_cast<const char*>(&*packet.begin()), packet.size(), &ts1, &ts2);
   std::string str;
-  message.serialize(str);
+  DnstapMessage message(str, DnstapMessage::MessageType::resolver_response, SyncRes::s_serverID, &localip, &ip, doTCP, static_cast<const char*>(&*packet.begin()), packet.size(), &ts1, &ts2, auth);
 
   for (auto& logger : *fstreamLoggers) {
     logger->queueData(str);
@@ -123,7 +118,7 @@ static void logFstreamResponse(const std::shared_ptr<std::vector<std::unique_ptr
 
 #endif // HAVE_FSTRM
 
-static void logOutgoingQuery(const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, boost::optional<RecProtoBufMessage>& message, boost::optional<const boost::uuids::uuid&> initialRequestId, const boost::uuids::uuid& uuid, const ComboAddress& ip, const DNSName& domain, int type, uint16_t qid, bool doTCP, size_t bytes, boost::optional<Netmask>& srcmask)
+static void logOutgoingQuery(const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, boost::optional<const boost::uuids::uuid&> initialRequestId, const boost::uuids::uuid& uuid, const ComboAddress& ip, const DNSName& domain, int type, uint16_t qid, bool doTCP, size_t bytes, boost::optional<Netmask>& srcmask)
 {
   if (!outgoingLoggers) {
     return;
@@ -141,29 +136,37 @@ static void logOutgoingQuery(const std::shared_ptr<std::vector<std::unique_ptr<R
     return;
   }
 
-  message = RecProtoBufMessage(DNSProtoBufMessage::OutgoingQuery, uuid, nullptr, &ip, domain, type, QClass::IN, qid, doTCP, bytes);
-  message->setServerIdentity(SyncRes::s_serverID);
+  static thread_local std::string buffer;
+  buffer.clear();
+  pdns::ProtoZero::Message m{buffer};
+  m.setType(pdns::ProtoZero::Message::MessageType::DNSOutgoingQueryType);
+  m.setMessageIdentity(uuid);
+  m.setSocketFamily(ip.sin4.sin_family);
+  m.setSocketProtocol(doTCP);
+  m.setTo(ip);
+  m.setInBytes(bytes);
+  m.setTime();
+  m.setId(qid);
+  m.setQuestion(domain, type, QClass::IN);
+  m.setToPort(ip.getPort());
+  m.setServerIdentity(SyncRes::s_serverID);
 
   if (initialRequestId) {
-    message->setInitialRequestID(*initialRequestId);
+    m.setInitialRequestID(*initialRequestId);
   }
 
   if (srcmask) {
-    message->setEDNSSubnet(*srcmask);
+    m.setEDNSSubnet(*srcmask, 128);
   }
 
-//  cerr <<message.toDebugString()<<endl;
-  std::string str;
-  message->serialize(str);
-
   for (auto& logger : *outgoingLoggers) {
     if (logger->logQueries()) {
-      logger->queueData(str);
+      logger->queueData(buffer);
     }
   }
 }
 
-static void logIncomingResponse(const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, boost::optional<RecProtoBufMessage>& message, boost::optional<const boost::uuids::uuid&> initialRequestId, const boost::uuids::uuid& uuid, const ComboAddress& ip, const DNSName& domain, int type, uint16_t qid, bool doTCP, boost::optional<Netmask>& srcmask, size_t bytes, int rcode, const std::vector<DNSRecord>& records, const struct timeval& queryTime, const std::set<uint16_t>& exportTypes)
+static void logIncomingResponse(const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, boost::optional<const boost::uuids::uuid&> initialRequestId, const boost::uuids::uuid& uuid, const ComboAddress& ip, const DNSName& domain, int type, uint16_t qid, bool doTCP, boost::optional<Netmask>& srcmask, size_t bytes, int rcode, const std::vector<DNSRecord>& records, const struct timeval& queryTime, const std::set<uint16_t>& exportTypes)
 {
   if (!outgoingLoggers) {
     return;
@@ -181,50 +184,54 @@ static void logIncomingResponse(const std::shared_ptr<std::vector<std::unique_pt
     return;
   }
 
-  if (!message) {
-    message = RecProtoBufMessage(DNSProtoBufMessage::IncomingResponse, uuid, nullptr, &ip, domain, type, QClass::IN, qid, doTCP, bytes);
-    message->setServerIdentity(SyncRes::s_serverID);
+  static thread_local std::string buffer;
+  buffer.clear();
+  pdns::ProtoZero::RecMessage m{buffer};
+  m.setType(pdns::ProtoZero::Message::MessageType::DNSIncomingResponseType);
+  m.setMessageIdentity(uuid);
+  m.setSocketFamily(ip.sin4.sin_family);
+  m.setSocketProtocol(doTCP);
+  m.setTo(ip);
+  m.setInBytes(bytes);
+  m.setTime();
+  m.setId(qid);
+  m.setQuestion(domain, type, QClass::IN);
+  m.setToPort(ip.getPort());
+  m.setServerIdentity(SyncRes::s_serverID);
 
-    if (initialRequestId) {
-      message->setInitialRequestID(*initialRequestId);
-    }
-
-    if (srcmask) {
-      message->setEDNSSubnet(*srcmask);
-    }
-  }
-  else {
-    message->updateTime();
-    message->setType(DNSProtoBufMessage::IncomingResponse);
-    message->setBytes(bytes);
+  if (initialRequestId) {
+    m.setInitialRequestID(*initialRequestId);
   }
 
-  message->setQueryTime(queryTime.tv_sec, queryTime.tv_usec);
+  if (srcmask) {
+    m.setEDNSSubnet(*srcmask, 128);
+  }
+
+  m.startResponse();
+  m.setQueryTime(queryTime.tv_sec, queryTime.tv_usec);
   if (rcode == -1) {
-    message->setNetworkErrorResponseCode();
+    m.setNetworkErrorResponseCode();
   }
   else {
-    message->setResponseCode(rcode);
+    m.setResponseCode(rcode);
   }
-  message->addRRs(records, exportTypes);
 
-//  cerr <<message.toDebugString()<<endl;
-  std::string str;
-  message->serialize(str);
+  for (const auto& record : records) {
+    m.addRR(record, exportTypes, false);
+  }
+  m.commitResponse();
 
   for (auto& logger : *outgoingLoggers) {
     if (logger->logResponses()) {
-      logger->queueData(str);
+      logger->queueData(buffer);
     }
   }
 }
-#endif /* HAVE_PROTOBUF */
 
-//! returns -2 for OS limits error, -1 for permanent error that has to do with remote **transport**, 0 for timeout, 1 for success
 /** lwr is only filled out in case 1 was returned, and even when returning 1 for 'success', lwr might contain DNS errors
     Never throws! 
  */
-int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool doTCP, bool sendRDQuery, int EDNS0Level, struct timeval* now, boost::optional<Netmask>& srcmask, boost::optional<const ResolveContext&> context, const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstrmLoggers, const std::set<uint16_t>& exportTypes, LWResult *lwr, bool* chained)
+LWResult::Result asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool doTCP, bool sendRDQuery, int EDNS0Level, struct timeval* now, boost::optional<Netmask>& srcmask, boost::optional<const ResolveContext&> context, const std::shared_ptr<std::vector<std::unique_ptr<RemoteLogger>>>& outgoingLoggers, const std::shared_ptr<std::vector<std::unique_ptr<FrameStreamLogger>>>& fstrmLoggers, const std::set<uint16_t>& exportTypes, LWResult *lwr, bool* chained)
 {
   size_t len;
   size_t bufsize=g_outgoingEDNSBufsize;
@@ -270,109 +277,140 @@ int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool d
   }
   lwr->d_rcode = 0;
   lwr->d_haveEDNS = false;
-  int ret;
+  LWResult::Result ret;
 
   DTime dt;
   dt.set();
   *now=dt.getTimeval();
 
-#ifdef HAVE_PROTOBUF
   boost::uuids::uuid uuid;
   const struct timeval queryTime = *now;
-  boost::optional<RecProtoBufMessage> pbMessage = boost::none;
 
   if (outgoingLoggers) {
     uuid = getUniqueID();
-    logOutgoingQuery(outgoingLoggers, pbMessage, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, vpacket.size(), srcmask);
+    logOutgoingQuery(outgoingLoggers, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, vpacket.size(), srcmask);
   }
-#endif /* HAVE_PROTOBUF */
-#ifdef HAVE_FSTRM
-  if (isEnabledForQueries(fstrmLoggers)) {
-    logFstreamQuery(fstrmLoggers, queryTime, ip, doTCP, context ? context->d_auth : boost::none, vpacket);
-  }
-#endif /* HAVE_FSTRM */
 
   srcmask = boost::none; // this is also our return value, even if EDNS0Level == 0
 
+  // We only store the localip if needed for fstrm logging
+  ComboAddress localip;
+  bool fstrmQEnabled = false;
+  bool fstrmREnabled = false;
+#ifdef HAVE_FSTRM
+  if (isEnabledForQueries(fstrmLoggers)) {
+    fstrmQEnabled = true;
+  }
+  if (isEnabledForResponses(fstrmLoggers)) {
+    fstrmREnabled = true;
+  }
+#endif
+
   if(!doTCP) {
     int queryfd;
-    if(ip.sin4.sin_family==AF_INET6)
+    if (ip.sin4.sin_family==AF_INET6) {
       g_stats.ipv6queries++;
+    }
 
-    if((ret=asendto((const char*)&*vpacket.begin(), vpacket.size(), 0, ip, qid,
-                    domain, type, &queryfd)) < 0) {
-      return ret; // passes back the -2 EMFILE
+    ret = asendto((const char*)&*vpacket.begin(), vpacket.size(), 0, ip, qid, domain, type, &queryfd);
+
+    if (ret != LWResult::Result::Success) {
+      return ret;
     }
 
     if (queryfd == -1) {
       *chained = true;
     }
 
+#ifdef HAVE_FSTRM
+    if (!*chained) {
+      if (fstrmQEnabled || fstrmREnabled) {
+        localip.sin4.sin_family = ip.sin4.sin_family;
+        socklen_t slen = ip.getSocklen();
+        getsockname(queryfd, reinterpret_cast<sockaddr*>(&localip), &slen);
+      }
+      if (fstrmQEnabled) {
+        logFstreamQuery(fstrmLoggers, queryTime, localip, ip, doTCP, context ? context->d_auth : boost::none, vpacket);
+      }
+    }
+#endif /* HAVE_FSTRM */
+
     // sleep until we see an answer to this, interface to mtasker
-    
-    ret=arecvfrom(buf, 0, ip, &len, qid,
-                  domain, type, queryfd, now);
+    ret = arecvfrom(buf, 0, ip, &len, qid, domain, type, queryfd, now);
   }
   else {
     try {
       Socket s(ip.sin4.sin_family, SOCK_STREAM);
 
       s.setNonBlocking();
-      ComboAddress local = pdns::getQueryLocalAddress(ip.sin4.sin_family, 0);
+      if (SyncRes::s_tcp_fast_open_connect) {
+        try {
+          s.setFastOpenConnect();
+        }
+        catch (const NetworkError& e) {
+          // Ignore error, we did a pre-check in pdns_recursor.cc:checkTFOconnect()
+        }
+      }
+
+      localip = pdns::getQueryLocalAddress(ip.sin4.sin_family, 0);
+      s.bind(localip);
 
-      s.bind(local);
-        
       s.connect(ip);
-      
+
       uint16_t tlen=htons(vpacket.size());
       char *lenP=(char*)&tlen;
       const char *msgP=(const char*)&*vpacket.begin();
       string packet=string(lenP, lenP+2)+string(msgP, msgP+vpacket.size());
-      
-      ret=asendtcp(packet, &s);
-      if(!(ret>0))           
+      ret = asendtcp(packet, &s);
+      if (ret != LWResult::Result::Success) {
         return ret;
-      
+      }
+
+#ifdef HAVE_FSTRM
+  if (fstrmQEnabled) {
+    logFstreamQuery(fstrmLoggers, queryTime, localip, ip, doTCP, context ? context->d_auth : boost::none, vpacket);
+  }
+#endif /* HAVE_FSTRM */
+
       packet.clear();
-      ret=arecvtcp(packet, 2, &s, false);
-      if(!(ret > 0))
+      ret = arecvtcp(packet, 2, &s, false);
+      if (ret != LWResult::Result::Success) {
         return ret;
-      
+      }
+
       memcpy(&tlen, packet.c_str(), sizeof(tlen));
       len=ntohs(tlen); // switch to the 'len' shared with the rest of the function
-      
-      ret=arecvtcp(packet, len, &s, false);
-      if(!(ret > 0))
+
+      ret = arecvtcp(packet, len, &s, false);
+      if (ret != LWResult::Result::Success) {
         return ret;
-      
+      }
+
       buf.resize(len);
       memcpy(const_cast<char*>(buf.data()), packet.c_str(), len);
 
-      ret=1;
+      ret = LWResult::Result::Success;
     }
-    catch(NetworkError& ne) {
-      ret = -2; // OS limits error
+    catch (const NetworkError& ne) {
+      ret = LWResult::Result::OSLimitError; // OS limits error
     }
   }
 
-  
   lwr->d_usec=dt.udiff();
   *now=dt.getTimeval();
 
-  if(ret <= 0) { // includes 'timeout'
-#ifdef HAVE_PROTOBUF
+  if (ret != LWResult::Result::Success) { // includes 'timeout'
       if (outgoingLoggers) {
-        logIncomingResponse(outgoingLoggers, pbMessage, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, 0, -1, {}, queryTime, exportTypes);
+        logIncomingResponse(outgoingLoggers, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, 0, -1, {}, queryTime, exportTypes);
       }
-#endif
     return ret;
   }
 
   buf.resize(len);
 
 #ifdef HAVE_FSTRM
-  if (isEnabledForResponses(fstrmLoggers)) {
-    logFstreamResponse(fstrmLoggers, ip, doTCP, context ? context->d_auth : boost::none, buf, queryTime, *now);
+  if (fstrmREnabled && (!*chained || doTCP)) {
+    logFstreamResponse(fstrmLoggers, localip, ip, doTCP, context ? context->d_auth : boost::none, buf, queryTime, *now);
   }
 #endif /* HAVE_FSTRM */
 
@@ -385,13 +423,11 @@ int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool d
     lwr->d_rcode=mdp.d_header.rcode;
     
     if(mdp.d_header.rcode == RCode::FormErr && mdp.d_qname.empty() && mdp.d_qtype == 0 && mdp.d_qclass == 0) {
-#ifdef HAVE_PROTOBUF
       if(outgoingLoggers) {
-        logIncomingResponse(outgoingLoggers, pbMessage, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
+        logIncomingResponse(outgoingLoggers, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
       }
-#endif
-      lwr->d_validpacket=true;
-      return 1; // this is "success", the error is set in lwr->d_rcode
+      lwr->d_validpacket = true;
+      return LWResult::Result::Success; // this is "success", the error is set in lwr->d_rcode
     }
 
     if(domain != mdp.d_qname) { 
@@ -431,37 +467,39 @@ int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool d
       }
     }
         
-#ifdef HAVE_PROTOBUF
     if(outgoingLoggers) {
-      logIncomingResponse(outgoingLoggers, pbMessage, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
+      logIncomingResponse(outgoingLoggers, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
     }
-#endif
-    lwr->d_validpacket=true;
-    return 1;
+
+    lwr->d_validpacket = true;
+    return LWResult::Result::Success;
   }
-  catch(std::exception &mde) {
-    if(::arg().mustDo("log-common-errors"))
+  catch (const std::exception &mde) {
+    if (::arg().mustDo("log-common-errors")) {
       g_log<<Logger::Notice<<"Unable to parse packet from remote server "<<ip.toString()<<": "<<mde.what()<<endl;
-    lwr->d_rcode = RCode::FormErr;
-    g_stats.serverParseError++;
-#ifdef HAVE_PROTOBUF
-    if(outgoingLoggers) {
-      logIncomingResponse(outgoingLoggers, pbMessage, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
     }
-#endif
-    lwr->d_validpacket=false;
-    return 1; // success - oddly enough
+
+    lwr->d_rcode = RCode::FormErr;
+    lwr->d_validpacket = false;
+    g_stats.serverParseError++;
+
+    if(outgoingLoggers) {
+      logIncomingResponse(outgoingLoggers, context ? context->d_initialRequestId : boost::none, uuid, ip, domain, type, qid, doTCP, srcmask, len, lwr->d_rcode, lwr->d_records, queryTime, exportTypes);
+    }
+
+    return LWResult::Result::Success; // success - oddly enough
   }
-  catch(...) {
+  catch (...) {
     g_log<<Logger::Notice<<"Unknown error parsing packet from remote server"<<endl;
   }
   
   g_stats.serverParseError++; 
   
  out:
-  if(!lwr->d_rcode)
+  if (!lwr->d_rcode) {
     lwr->d_rcode=RCode::ServFail;
+  }
 
-  return -1;
+  return LWResult::Result::PermanentError;
 }
 
