@@ -53,12 +53,12 @@ typename C::value_type::second_type constGet(const C& c, const std::string& name
 
 typedef std::unordered_map<std::string, boost::variant<bool, uint32_t, std::string, std::vector<std::pair<int, std::string>> > > rpzOptions_t;
 
-static void parseRPZParameters(rpzOptions_t& have, std::string& polName, boost::optional<DNSFilterEngine::Policy>& defpol, bool& defpolOverrideLocal, uint32_t& maxTTL, size_t& zoneSizeHint, std::unordered_set<std::string>& tags, bool& overridesGettag)
+static void parseRPZParameters(rpzOptions_t& have, std::shared_ptr<DNSFilterEngine::Zone>& zone, std::string& polName, boost::optional<DNSFilterEngine::Policy>& defpol, bool& defpolOverrideLocal, uint32_t& maxTTL)
 {
-  if(have.count("policyName")) {
+  if (have.count("policyName")) {
     polName = boost::get<std::string>(have["policyName"]);
   }
-  if(have.count("defpol")) {
+  if (have.count("defpol")) {
     defpol=DNSFilterEngine::Policy();
     defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<uint32_t>(have["defpol"]);
     defpol->setName(polName);
@@ -76,24 +76,39 @@ static void parseRPZParameters(rpzOptions_t& have, std::string& polName, boost::
       defpolOverrideLocal = boost::get<bool>(have["defpolOverrideLocalData"]);
     }
   }
-  if(have.count("maxTTL")) {
+  if (have.count("maxTTL")) {
     maxTTL = boost::get<uint32_t>(have["maxTTL"]);
   }
-  if(have.count("zoneSizeHint")) {
-    zoneSizeHint = static_cast<size_t>(boost::get<uint32_t>(have["zoneSizeHint"]));
+  if (have.count("zoneSizeHint")) {
+    auto zoneSizeHint = static_cast<size_t>(boost::get<uint32_t>(have["zoneSizeHint"]));
+    if (zoneSizeHint > 0) {
+      zone->reserve(zoneSizeHint);
+    }
   }
   if (have.count("tags")) {
     const auto tagsTable = boost::get<std::vector<std::pair<int, std::string>>>(have["tags"]);
+    std::unordered_set<std::string> tags;
     for (const auto& tag : tagsTable) {
       tags.insert(tag.second);
     }
+    zone->setTags(std::move(tags));
   }
   if (have.count("overridesGettag")) {
-    overridesGettag = boost::get<bool>(have["overridesGettag"]);
+    zone->setPolicyOverridesGettag(boost::get<bool>(have["overridesGettag"]));
+  }
+  if (have.count("extendedErrorCode")) {
+    auto code = boost::get<uint32_t>(have["extendedErrorCode"]);
+    if (code > std::numeric_limits<uint16_t>::max()) {
+      throw std::runtime_error("Invalid extendedErrorCode value " + std::to_string(code) + " in RPZ configuration");
+    }
+
+    zone->setExtendedErrorCode(static_cast<uint16_t>(code));
+    if (have.count("extendedErrorExtra")) {
+      zone->setExtendedErrorExtra(boost::get<std::string>(have["extendedErrorExtra"]));
+    }
   }
 }
 
-#if HAVE_PROTOBUF
 typedef std::unordered_map<std::string, boost::variant<bool, uint64_t, std::string, std::vector<std::pair<int,std::string> > > > protobufOptions_t;
 
 static void parseProtobufOptions(boost::optional<protobufOptions_t> vars, ProtobufExportConfig& config)
@@ -152,7 +167,6 @@ static void parseProtobufOptions(boost::optional<protobufOptions_t> vars, Protob
     }
   }
 }
-#endif /* HAVE_PROTOBUF */
 
 #ifdef HAVE_FSTRM
 typedef std::unordered_map<std::string, boost::variant<bool, uint64_t, std::string, std::vector<std::pair<int,std::string> > > > frameStreamOptions_t;
@@ -191,6 +205,123 @@ static void parseFrameStreamOptions(boost::optional<frameStreamOptions_t> vars,
 }
 #endif /* HAVE_FSTRM */
 
+static void rpzPrimary(LuaConfigItems& lci, luaConfigDelayedThreads& delayedThreads, const boost::variant<string, std::vector<std::pair<int, string> > >& masters_, const string& zoneName, boost::optional<rpzOptions_t> options)
+{
+  boost::optional<DNSFilterEngine::Policy> defpol;
+  bool defpolOverrideLocal = true;
+  std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
+  TSIGTriplet tt;
+  uint32_t refresh=0;
+  size_t maxReceivedXFRMBytes = 0;
+  uint16_t axfrTimeout = 20;
+  uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
+  ComboAddress localAddress;
+  std::vector<ComboAddress> masters;
+  if (masters_.type() == typeid(string)) {
+    masters.push_back(ComboAddress(boost::get<std::string>(masters_), 53));
+  }
+  else {
+    for (const auto& master : boost::get<std::vector<std::pair<int, std::string>>>(masters_)) {
+      masters.push_back(ComboAddress(master.second, 53));
+    }
+  }
+
+  size_t zoneIdx;
+  std::string dumpFile;
+  std::shared_ptr<SOARecordContent> sr = nullptr;
+
+  try {
+    std::string seedFile;
+    std::string polName(zoneName);
+
+    if (options) {
+      auto& have = *options;
+      parseRPZParameters(have, zone, polName, defpol, defpolOverrideLocal, maxTTL);
+
+      if(have.count("tsigname")) {
+        tt.name=DNSName(toLower(boost::get<string>(have["tsigname"])));
+        tt.algo=DNSName(toLower(boost::get<string>(have[ "tsigalgo"])));
+        if(B64Decode(boost::get<string>(have[ "tsigsecret"]), tt.secret))
+          throw std::runtime_error("TSIG secret is not valid Base-64 encoded");
+      }
+
+      if(have.count("refresh")) {
+        refresh = boost::get<uint32_t>(have["refresh"]);
+        if (refresh == 0) {
+          g_log<<Logger::Warning<<"rpzPrimary refresh value of 0 ignored"<<endl;
+        }
+      }
+
+      if(have.count("maxReceivedMBytes")) {
+        maxReceivedXFRMBytes = static_cast<size_t>(boost::get<uint32_t>(have["maxReceivedMBytes"]));
+      }
+
+      if(have.count("localAddress")) {
+        localAddress = ComboAddress(boost::get<string>(have["localAddress"]));
+      }
+
+      if(have.count("axfrTimeout")) {
+        axfrTimeout = static_cast<uint16_t>(boost::get<uint32_t>(have["axfrTimeout"]));
+      }
+
+      if(have.count("seedFile")) {
+        seedFile = boost::get<std::string>(have["seedFile"]);
+      }
+
+      if(have.count("dumpFile")) {
+        dumpFile = boost::get<std::string>(have["dumpFile"]);
+      }
+    }
+
+    if (localAddress != ComboAddress()) {
+      // We were passed a localAddress, check if its AF matches the masters'
+      for (const auto& master : masters) {
+        if (localAddress.sin4.sin_family != master.sin4.sin_family) {
+          throw PDNSException("Primary address("+master.toString()+") is not of the same Address Family as the local address ("+localAddress.toString()+").");
+        }
+      }
+    }
+
+    DNSName domain(zoneName);
+    zone->setDomain(domain);
+    zone->setName(polName);
+    zoneIdx = lci.dfe.addZone(zone);
+
+    if (!seedFile.empty()) {
+      g_log<<Logger::Info<<"Pre-loading RPZ zone "<<zoneName<<" from seed file '"<<seedFile<<"'"<<endl;
+      try {
+        sr = loadRPZFromFile(seedFile, zone, defpol, defpolOverrideLocal, maxTTL);
+
+        if (zone->getDomain() != domain) {
+          throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") does not match the one passed in parameter (" + domain.toString() + ")");
+        }
+
+        if (sr == nullptr) {
+          throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") has no SOA record");
+        }
+      }
+      catch(const PDNSException& e) {
+        g_log<<Logger::Warning<<"Unable to pre-load RPZ zone "<<zoneName<<" from seed file '"<<seedFile<<"': "<<e.reason<<endl;
+        zone->clear();
+      }
+      catch(const std::exception& e) {
+        g_log<<Logger::Warning<<"Unable to pre-load RPZ zone "<<zoneName<<" from seed file '"<<seedFile<<"': "<<e.what()<<endl;
+        zone->clear();
+      }
+    }
+  }
+  catch(const std::exception& e) {
+    g_log<<Logger::Error<<"Problem configuring 'rpzPrimary': "<<e.what()<<endl;
+    exit(1);  // FIXME proper exit code?
+  }
+  catch(const PDNSException& e) {
+    g_log<<Logger::Error<<"Problem configuring 'rpzPrimary': "<<e.reason<<endl;
+    exit(1);  // FIXME proper exit code?
+  }
+
+  delayedThreads.rpzMasterThreads.push_back(std::make_tuple(masters, defpol, defpolOverrideLocal, maxTTL, zoneIdx, tt, maxReceivedXFRMBytes, localAddress, axfrTimeout, refresh, sr, dumpFile));
+}
+
 void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& delayedThreads)
 {
   LuaConfigItems lci;
@@ -244,20 +375,12 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
         std::string polName("rpzFile");
         std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
         uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
-        bool overridesGettag = true;
-        if(options) {
+        if (options) {
           auto& have = *options;
-          size_t zoneSizeHint = 0;
-          std::unordered_set<std::string> tags;
-          parseRPZParameters(have, polName, defpol, defpolOverrideLocal, maxTTL, zoneSizeHint, tags, overridesGettag);
-          if (zoneSizeHint > 0) {
-            zone->reserve(zoneSizeHint);
-          }
-          zone->setTags(std::move(tags));
+          parseRPZParameters(have, zone, polName, defpol, defpolOverrideLocal, maxTTL);
         }
         g_log<<Logger::Warning<<"Loading RPZ from file '"<<filename<<"'"<<endl;
         zone->setName(polName);
-        zone->setPolicyOverridesGettag(overridesGettag);
         loadRPZFromFile(filename, zone, defpol, defpolOverrideLocal, maxTTL);
         lci.dfe.addZone(zone);
         g_log<<Logger::Warning<<"Done loading RPZ from file '"<<filename<<"'"<<endl;
@@ -268,124 +391,11 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
     });
 
   Lua.writeFunction("rpzMaster", [&lci, &delayedThreads](const boost::variant<string, std::vector<std::pair<int, string> > >& masters_, const string& zoneName, boost::optional<rpzOptions_t> options) {
-
-      boost::optional<DNSFilterEngine::Policy> defpol;
-      bool defpolOverrideLocal = true;
-      std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
-      TSIGTriplet tt;
-      uint32_t refresh=0;
-      size_t maxReceivedXFRMBytes = 0;
-      uint16_t axfrTimeout = 20;
-      uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
-      ComboAddress localAddress;
-      std::vector<ComboAddress> masters;
-      if (masters_.type() == typeid(string)) {
-        masters.push_back(ComboAddress(boost::get<std::string>(masters_), 53));
-      }
-      else {
-        for (const auto& master : boost::get<std::vector<std::pair<int, std::string>>>(masters_)) {
-          masters.push_back(ComboAddress(master.second, 53));
-        }
-      }
-
-      size_t zoneIdx;
-      std::string dumpFile;
-      std::shared_ptr<SOARecordContent> sr = nullptr;
-
-      try {
-        std::string seedFile;
-        std::string polName(zoneName);
-
-        if (options) {
-          auto& have = *options;
-          size_t zoneSizeHint = 0;
-          std::unordered_set<std::string> tags;
-          bool overridesGettag = true;
-          parseRPZParameters(have, polName, defpol, defpolOverrideLocal, maxTTL, zoneSizeHint, tags, overridesGettag);
-          if (zoneSizeHint > 0) {
-            zone->reserve(zoneSizeHint);
-          }
-          zone->setTags(std::move(tags));
-          zone->setPolicyOverridesGettag(overridesGettag);
-
-          if(have.count("tsigname")) {
-            tt.name=DNSName(toLower(boost::get<string>(have["tsigname"])));
-            tt.algo=DNSName(toLower(boost::get<string>(have[ "tsigalgo"])));
-            if(B64Decode(boost::get<string>(have[ "tsigsecret"]), tt.secret))
-              throw std::runtime_error("TSIG secret is not valid Base-64 encoded");
-          }
-
-          if(have.count("refresh")) {
-            refresh = boost::get<uint32_t>(have["refresh"]);
-            if (refresh == 0) {
-              g_log<<Logger::Warning<<"rpzMaster refresh value of 0 ignored"<<endl;
-            }
-          }
-
-          if(have.count("maxReceivedMBytes")) {
-            maxReceivedXFRMBytes = static_cast<size_t>(boost::get<uint32_t>(have["maxReceivedMBytes"]));
-          }
-
-          if(have.count("localAddress")) {
-            localAddress = ComboAddress(boost::get<string>(have["localAddress"]));
-          }
-
-          if(have.count("axfrTimeout")) {
-            axfrTimeout = static_cast<uint16_t>(boost::get<uint32_t>(have["axfrTimeout"]));
-          }
-
-          if(have.count("seedFile")) {
-            seedFile = boost::get<std::string>(have["seedFile"]);
-          }
-
-          if(have.count("dumpFile")) {
-            dumpFile = boost::get<std::string>(have["dumpFile"]);
-          }
-        }
-
-        if (localAddress != ComboAddress()) {
-          // We were passed a localAddress, check if its AF matches the masters'
-          for (const auto& master : masters) {
-            if (localAddress.sin4.sin_family != master.sin4.sin_family) {
-              throw PDNSException("Master address("+master.toString()+") is not of the same Address Family as the local address ("+localAddress.toString()+").");
-            }
-          }
-        }
-
-        DNSName domain(zoneName);
-        zone->setDomain(domain);
-        zone->setName(polName);
-        zoneIdx = lci.dfe.addZone(zone);
-
-        if (!seedFile.empty()) {
-          g_log<<Logger::Info<<"Pre-loading RPZ zone "<<zoneName<<" from seed file '"<<seedFile<<"'"<<endl;
-          try {
-            sr = loadRPZFromFile(seedFile, zone, defpol, defpolOverrideLocal, maxTTL);
-
-            if (zone->getDomain() != domain) {
-              throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") does not match the one passed in parameter (" + domain.toString() + ")");
-            }
-
-            if (sr == nullptr) {
-              throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") has no SOA record");
-            }
-          }
-          catch(const std::exception& e) {
-            g_log<<Logger::Warning<<"Unable to pre-load RPZ zone "<<zoneName<<" from seed file '"<<seedFile<<"': "<<e.what()<<endl;
-          }
-        }
-      }
-      catch(const std::exception& e) {
-        g_log<<Logger::Error<<"Problem configuring 'rpzMaster': "<<e.what()<<endl;
-        exit(1);  // FIXME proper exit code?
-      }
-      catch(const PDNSException& e) {
-        g_log<<Logger::Error<<"Problem configuring 'rpzMaster': "<<e.reason<<endl;
-        exit(1);  // FIXME proper exit code?
-      }
-
-      delayedThreads.rpzMasterThreads.push_back(std::make_tuple(masters, defpol, defpolOverrideLocal, maxTTL, zoneIdx, tt, maxReceivedXFRMBytes, localAddress, axfrTimeout, refresh, sr, dumpFile));
-    });
+    rpzPrimary(lci, delayedThreads, masters_, zoneName, options);
+      });
+  Lua.writeFunction("rpzPrimary", [&lci, &delayedThreads](const boost::variant<string, std::vector<std::pair<int, string> > >& masters_, const string& zoneName, boost::optional<rpzOptions_t> options) {
+    rpzPrimary(lci, delayedThreads, masters_, zoneName, options);
+      });
 
   typedef vector<pair<int,boost::variant<string, vector<pair<int, string> > > > > argvec_t;
   Lua.writeFunction("addSortList", 
@@ -480,7 +490,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
       updateTrustAnchorsFromFile(fnamearg, lci.dsAnchors);
     });
 
-#if HAVE_PROTOBUF
   Lua.writeFunction("setProtobufMasks", [&lci](const uint8_t maskV4, uint8_t maskV6) {
       lci.protobufMaskV4 = maskV4;
       lci.protobufMaskV6 = maskV6;
@@ -549,7 +558,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
         g_log<<Logger::Error<<"Only one outgoingProtobufServer() directive can be configured, we already have "<<lci.outgoingProtobufExportConfig.servers.at(0).toString()<<endl;
       }
     });
-#endif
 
 #ifdef HAVE_FSTRM
   Lua.writeFunction("dnstapFrameStreamServer", [&lci](boost::variant<const std::string, const std::unordered_map<int, std::string>> servers, boost::optional<frameStreamOptions_t> vars) {
