@@ -48,7 +48,6 @@
 
 #include "dns_random.hh"
 #include <poll.h>
-#include "gss_context.hh"
 #include "namespaces.hh"
 
 using pdns::resolver::parseResult;
@@ -62,7 +61,7 @@ int makeQuerySocket(const ComboAddress& local, bool udpOrTCP, bool nonLocalBind)
     if(errno == EAFNOSUPPORT && local.sin4.sin_family == AF_INET6) {
         return -1;
     }
-    unixDie("Creating local resolver socket for "+ourLocal.toString());
+    unixDie("Creating local resolver socket for address '"+ourLocal.toString()+"'");
   }
 
   setCloseOnExec(sock);
@@ -83,7 +82,7 @@ int makeQuerySocket(const ComboAddress& local, bool udpOrTCP, bool nonLocalBind)
 
     if(!tries) {
       closesocket(sock);
-      throw PDNSException("Resolver binding to local UDP socket on "+ourLocal.toString()+": "+stringerror());
+      throw PDNSException("Resolver binding to local UDP socket on '"+ourLocal.toString()+"': "+stringerror());
     }
   }
   else {
@@ -91,7 +90,7 @@ int makeQuerySocket(const ComboAddress& local, bool udpOrTCP, bool nonLocalBind)
     ourLocal.sin4.sin_port = 0;
     if(::bind(sock, (struct sockaddr *)&ourLocal, ourLocal.getSocklen()) < 0) {
       closesocket(sock);
-      throw PDNSException("Resolver binding to local TCP socket on "+ourLocal.toString()+": "+stringerror());
+      throw PDNSException("Resolver binding to local TCP socket on '"+ourLocal.toString()+"': "+stringerror());
     }
   }
   return sock;
@@ -148,7 +147,7 @@ uint16_t Resolver::sendResolve(const ComboAddress& remote, const ComboAddress& l
       trc.d_algoName = tsigalgorithm + DNSName("sig-alg.reg.int");
     else
       trc.d_algoName = tsigalgorithm;
-    trc.d_time = time(0);
+    trc.d_time = time(nullptr);
     trc.d_fudge = 300;
     trc.d_origID=ntohs(randomid);
     trc.d_eRcode=0;
@@ -176,7 +175,7 @@ uint16_t Resolver::sendResolve(const ComboAddress& remote, const ComboAddress& l
       // try to make socket
       sock = makeQuerySocket(local, true);
       if (sock < 0)
-        throw ResolverException("Unable to create local socket on "+lstr+" to "+remote.toStringWithPort()+": "+stringerror());
+        throw ResolverException("Unable to create local socket on '"+lstr+"'' to '"+remote.toStringWithPort()+"': "+stringerror());
       setNonBlocking( sock );
       locals[lstr] = sock;
     }
@@ -186,7 +185,7 @@ uint16_t Resolver::sendResolve(const ComboAddress& remote, const ComboAddress& l
     *localsock = sock;
   }
   if(sendto(sock, &packet[0], packet.size(), 0, (struct sockaddr*)(&remote), remote.getSocklen()) < 0) {
-    throw ResolverException("Unable to ask query of "+remote.toStringWithPort()+": "+stringerror());
+    throw ResolverException("Unable to ask query of '"+remote.toStringWithPort()+"': "+stringerror());
   }
   return randomid;
 }
@@ -328,7 +327,7 @@ int Resolver::resolve(const ComboAddress& to, const DNSName &domain, int type, R
       throw ResolverException("recvfrom error waiting for answer: "+stringerror());
 
     if (from != to) {
-      throw ResolverException("Got answer from the wrong peer while resolving ("+from.toStringWithPort()+" instead of "+to.toStringWithPort()+", discarding");
+      throw ResolverException("Got answer from the wrong peer while resolving ('"+from.toStringWithPort()+"' instead of '"+to.toStringWithPort()+"', discarding");
     }
 
     MOADNSParser mdp(false, buffer, len);
