@@ -29,11 +29,18 @@ vState increaseDNSSECStateCounter(const vState& state)
   return state;
 }
 
+vState increaseXDNSSECStateCounter(const vState& state)
+{
+  g_stats.xdnssecResults[state]++;
+  return state;
+}
+
 // Returns true if dsAnchors were modified
 bool updateTrustAnchorsFromFile(const std::string &fname, map<DNSName, dsmap_t> &dsAnchors) {
   map<DNSName,dsmap_t> newDSAnchors;
   try {
     auto zp = ZoneParserTNG(fname);
+    zp.disableGenerate();
     DNSResourceRecord rr;
     DNSRecord dr;
     while(zp.get(rr)) {
@@ -50,7 +57,7 @@ bool updateTrustAnchorsFromFile(const std::string &fname, map<DNSName, dsmap_t>
         if (dnskeyr == nullptr) {
           throw PDNSException("Unable to parse DNSKEY record '" + rr.qname.toString() + " " + rr.getZoneRepresentation() +"'");
         }
-        auto dsr = makeDSFromDNSKey(rr.qname, *dnskeyr, DNSSECKeeper::SHA256);
+        auto dsr = makeDSFromDNSKey(rr.qname, *dnskeyr, DNSSECKeeper::DIGEST_SHA256);
         newDSAnchors[rr.qname].insert(dsr);
       }
     }
